package de.mcs.tools.sps.mnemonic;

import static org.junit.Assert.assertFalse;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import de.mcs.tools.sps.exceptions.IllegalArgument;
import de.mcs.tools.sps.exceptions.SyntaxError;

class TestARC {

  private static final int BYTE_VALUE = 0x6b;
  private final String MNEMONIC = "ARC";

  @Test
  void testMnemonic() throws SyntaxError {
    ARC mnemonic = new ARC(MNEMONIC + " 0x01");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE, mnemonic.getByte());
    assertTrue(mnemonic.hasArgument());
    assertFalse(mnemonic.isUsingPage());

    mnemonic = new ARC(MNEMONIC + " 0x02");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE + 1, mnemonic.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      ARC mno1 = new ARC(MNEMONIC + " 0x03");
      mno1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      ARC mno1 = new ARC(MNEMONIC + " 0");
      mno1.checkArgument();
    });
  }

  @Test
  void testMnemonicFactory() throws SyntaxError {
    Mnemonic mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 1", 0);
    assertEquals(ARC.class, mnemonic.getClass());

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC, 0);
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC + " 0x00", 0);
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC + " 5", 0);
    });

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 0x02", 0);
    assertEquals(BYTE_VALUE + 0x01, mnemonic.getByte());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 1", 0);
    assertEquals(BYTE_VALUE, mnemonic.getByte());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 0b00000010", 0);
    assertEquals(BYTE_VALUE + 1, mnemonic.getByte());

  }
}
