package de.mcs.tools.sps.mnemonic;

import static org.junit.Assert.assertFalse;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import de.mcs.tools.sps.exceptions.IllegalArgument;
import de.mcs.tools.sps.exceptions.SyntaxError;

class TestDIN {

  private static final int BYTE_VALUE = 0x64;
  private final String MNEMONIC = "DIN";

  @Test
  void testMnemonic() throws SyntaxError {
    DIN mnemonic = new DIN(MNEMONIC);
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE, mnemonic.getByte());
    assertTrue(mnemonic.hasArgument());
    assertFalse(mnemonic.isUsingPage());

    mnemonic = new DIN(MNEMONIC + " 0x01");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE + 1, mnemonic.getByte());

    mnemonic = new DIN(MNEMONIC + " 0x04");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE + 0x04, mnemonic.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      DIN mno1 = new DIN(MNEMONIC + " 0x05");
      mno1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      DIN mno1 = new DIN(MNEMONIC + " 0x10");
      mno1.checkArgument();
    });
  }

  @Test
  void testMnemonicFactory() throws SyntaxError {
    Mnemonic mnemonic = MnemonicFactory.getMnemonic(MNEMONIC, 0);
    assertEquals(DIN.class, mnemonic.getClass());

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC + " 0x00", 0);
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC + " 5", 0);
    });

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 0x02", 0);
    assertEquals(BYTE_VALUE + 0x02, mnemonic.getByte());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 3", 0);
    assertEquals(BYTE_VALUE + 3, mnemonic.getByte());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 0b00000100", 0);
    assertEquals(BYTE_VALUE + 4, mnemonic.getByte());

  }
}
