package de.mcs.tools.sps.mnemonic;

import static org.junit.Assert.assertFalse;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import de.mcs.tools.sps.exceptions.IllegalArgument;
import de.mcs.tools.sps.exceptions.SyntaxError;

class TestRJMP {

  private static final int BYTE_VALUE = 0x30;
  private final String MNEMONIC = "RJMP";

  @Test
  void testMnemonic() throws SyntaxError {
    RJMP mnemonic = new RJMP(MNEMONIC + " 0x00");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE, mnemonic.getByte());
    assertTrue(mnemonic.hasArgument());
    assertFalse(mnemonic.isUsingPage());

    mnemonic = new RJMP(MNEMONIC + " 0x01");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE + 1, mnemonic.getByte());

    mnemonic = new RJMP(MNEMONIC + " 0x0f");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE + 0x0f, mnemonic.getByte());

    mnemonic = new RJMP(MNEMONIC + " :test");
    mnemonic.checkArgument();
    assertTrue(mnemonic.isLabel());

    Assertions.assertThrows(SyntaxError.class, () -> {
      RJMP rjmp1 = new RJMP(MNEMONIC);
      rjmp1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      RJMP rjmp1 = new RJMP(MNEMONIC + " 0x10");
      rjmp1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      RJMP rjmp1 = new RJMP(MNEMONIC + " sieben");
      rjmp1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      RJMP rjmp1 = new RJMP(MNEMONIC + " 12ms");
      rjmp1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      RJMP rjmp1 = new RJMP(MNEMONIC + " 50s");
      rjmp1.checkArgument();
    });
  }

  @Test
  void testMnemonicFactory() throws SyntaxError {
    Mnemonic mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 0x00", 0);
    assertEquals(RJMP.class, mnemonic.getClass());

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC, 0);
    });

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 0x02", 0);
    assertEquals(BYTE_VALUE + 0x02, mnemonic.getByte());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 12", 0);
    assertEquals(BYTE_VALUE + 12, mnemonic.getByte());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 0b00001010", 0);
    assertEquals(BYTE_VALUE + 10, mnemonic.getByte());

  }
}
